import datetime
import json

import fitbit

from tests.config import INTRADAY_RESPONSES_FOLDER, RESPONSES_FOLDER


class FakeFitbit(fitbit.Fitbit):
    def __init__(
        self,
        client_id,
        client_secret,
        access_token=None,
        refresh_token=None,
        expires_at=None,
        refresh_cb=None,
        redirect_uri=None,
        system="en_US",
        **kwargs,
    ):
        # Override init as it does weird dynamic things that override
        # some of the methods defined here
        pass

    def intraday_time_series(
        self,
        resource: str,
        base_date="today",
        detail_level="1min",
        start_time=None,
        end_time=None,
    ) -> dict:
        resource_parts = resource.split("/")
        if len(resource_parts) != 2:
            raise ValueError(f"could not parse resource {resource}")
        dataset = resource_parts[1]
        return _load_intraday_response_data(dataset)

    def activities(self, date: datetime.date) -> dict:
        return _load_normal_response_data("activity")

    def get_sleep(self, date: datetime.date) -> dict:
        return _load_normal_response_data("sleep")


def _load_intraday_response_data(name: str) -> dict:
    path = INTRADAY_RESPONSES_FOLDER / f"{name}.json"
    return json.loads(path.read_text())


def _load_normal_response_data(name: str) -> dict:
    path = RESPONSES_FOLDER / f"{name}.json"
    return json.loads(path.read_text())



import pytest

from fitbit_downloader.config import Config, OAuthConfig, DownloadConfig
from tests.config import GENERATED_PATH


@pytest.fixture
def config() -> Config:
    return Config(
        client_id="a",
        client_secret="b",
        oauth_config=OAuthConfig(access_token="c", refresh_token="d", expires_at=123),
        download=DownloadConfig(fs_url=str(GENERATED_PATH)),
    )


import filecmp
import os
import shutil
from pathlib import Path
from typing import Union

from tests.config import GENERATED_PATH


def assert_dir_trees_are_equal(dir1: Union[str, Path], dir2: Union[str, Path]):
    """
    Compare two directories recursively. Files in each directory are
    assumed to be equal if their names and contents are equal.

    See: https://stackoverflow.com/a/6681395/6276321

    @param dir1: First directory path
    @param dir2: Second directory path

    @return: True if the directory trees are the same and
        there were no errors while accessing the directories or files,
        False otherwise.
    """

    dirs_cmp = filecmp.dircmp(dir1, dir2)
    if (
        len(dirs_cmp.left_only) > 0
        or len(dirs_cmp.right_only) > 0
        or len(dirs_cmp.funny_files) > 0
    ):
        raise AssertionError(
            f"lefy only: {dirs_cmp.left_only}. right only: {dirs_cmp.right_only}. funny files: {dirs_cmp.funny_files}"
        )
    (_, mismatch, errors) = filecmp.cmpfiles(
        dir1, dir2, dirs_cmp.common_files, shallow=False
    )
    if len(mismatch) > 0 or len(errors) > 0:
        raise AssertionError(f"mismatch: {mismatch}. errors: {errors}")
    for common_dir in dirs_cmp.common_dirs:
        new_dir1 = os.path.join(dir1, common_dir)
        new_dir2 = os.path.join(dir2, common_dir)
        assert_dir_trees_are_equal(new_dir1, new_dir2)


def reset_generated_path():
    if GENERATED_PATH.exists():
        shutil.rmtree(GENERATED_PATH)
    GENERATED_PATH.mkdir()


import fitbit
import pytest
from freezegun import freeze_time

from fitbit_downloader.config import Config
from fitbit_downloader.download import download_data
from tests.config import GENERATED_PATH, DOWNLOAD_INPUT_PATH
from tests.dirutils import assert_dir_trees_are_equal, reset_generated_path
from tests.fake_client import FakeFitbit


@pytest.fixture(autouse=True)
def fake_fitbit(monkeypatch):
    monkeypatch.setattr(fitbit, "Fitbit", FakeFitbit)


@freeze_time("2021-11-14")
def test_download(config: Config):
    reset_generated_path()
    download_data(config)
    assert_dir_trees_are_equal(GENERATED_PATH, DOWNLOAD_INPUT_PATH)
